// -*- c++ -*-

#ifndef __KDE_KDOUBLESPINBOX_H__
#define __KDE_KDOUBLESPINBOX_H__

#include <qspinbox.h>

/**
   @short A spin box for floating point values.
   @author Marc Mutz <mutz@kde.org>
   @version $Id: $
**/

class KDoubleSpinBox : public QSpinBox {
  Q_OBJECT
public:
  /** Constructs a @ref KDoubleSpinBox with parent @p parent and
      default values for range and value (whatever @ref QRangeControl
      uses) and precision (1). */
  KDoubleSpinBox( QWidget * parent=0, const char * name=0 );
  /** Constructs a @ref KDoubleSpinBox with parent @p parent, range
      [@p lower,@p upper], @ref lineStep @p step, @ref precision @p
      precision and initial value @p value. */
  KDoubleSpinBox( double lower, double upper, double step, double value,
		  int precision=1, QWidget * parent=0, const char * name=0 );

  virtual ~KDoubleSpinBox();

  /** @return whether the spinbx uses localized numbers */
  bool acceptLocalizedNumbers() const;
  /** Sets whether to use and accept localized numbers as returned by
      @ref KLocale::formatNumber() */
  virtual void setAcceptLocalizedNumbers( bool accept );

  /** Sets a new range for the spin box values. Note that @p lower, @p
      upper and @p step are rounded to @p precision decimal points
      first. */
  void setRange( double lower, double upper, double step, int precision );

  /** @return the current number of decimal points displayed. */
  int precision() const;
  /** Sets the number of decimal points to use. Note that there is a
      tradeoff between the precision used and the available range of
      values. See the class docs for more.
      @param precision the new number of decimal points to use

      @param force disables checking of bound violations that can
             arise if you increase the precision so much that the
             minimum and maximum values can't be represented
             anymore. Disabling is useful if you don't want to keep
             the current min and max values anyway. This is what
             e.g. @ref setRange() does.
  **/
  virtual void setPrecision( int precision, bool force=false );

  /** @return the cuurent value */
  double value() const;
  /** @return the current lower bound */
  double minValue() const;
  /** Sets the lower bound of the range to @p value, subject to the
      contraints that @p value is first rounded to the current
      precision and then clipped to the maximum representable
      interval.
      @see maxValue, minValue, setMaxValue, setRange
  */
  void setMinValue( double value );
  /** @return the current upper bound */
  double maxValue() const;
  /** Sets the upper bound of the range to @p value, subject to the
      contraints that @p value is first rounded to the current
      precision and then clipped to the maximum representable
      interval.
      @see minValue, maxValue, setMinValue, setRange
  */
  void setMaxValue( double value );

  /** @return the current step size */
  double lineStep() const;
  /** Sets the step size for clicking the up/down buttons to @p step,
      subject to the constraints that @p step is first rounded to the
      current precision and then clipped to the meaningful interval
      [1, @p maxValue - @p minValue]. */
  void setLineStep( double step );

  /** Overridden to ignore any setValidator() calls. */
  void setValidator( const QValidator * );

signals:
  /** Emitted whenever @ref QSpinBox::valueChanged( int ) is emitted. */
  virtual void valueChanged( double value );

public slots:
  /** Sets the current value to @p value, cubject to the constraints
      that @p value is frist rounded to the current precision and then
      clipped to the interval [@p minvalue(),@p maxValue()]. */
  virtual void setValue( double value );

protected:
  virtual QString mapValueToText(int);
  virtual int mapTextToValue(bool*);

protected slots:
  void slotValueChanged( int value );

private:
  typedef QSpinBox base; 
  void updateValidator();
  int maxPrecision() const;

  class Private;
  Private * d;
};

#endif // __KDE_KDOUBLESPINBOX_H__
